﻿namespace Brizco.Repository.Handlers.Activities;

public class CreateActivityCommandHandler(IRepositoryWrapper repositoryWrapper, ICurrentUserService currentUserService)
    : IRequestHandler<CreateActivityCommand, ActivityLDto>
{
    public async Task<ActivityLDto> Handle(CreateActivityCommand request, CancellationToken cancellationToken)
    {
        if (currentUserService.ComplexId == null)
            throw new AppException("ComplexId is null", ApiResultStatusCode.NotFound);
        if (!Guid.TryParse(currentUserService.ComplexId, out Guid complexId))
            throw new AppException("ComplexId is wrong", ApiResultStatusCode.NotFound);
        try
        {
            await repositoryWrapper.BeginTransaction(cancellationToken);
            var activity = Domain.Entities.Tasks.Activity
                .Create(
                    ActivityStatus.Created,
                    DateTime.MinValue,
                    string.Empty,
                    request.Title,
                    request.Description,
                    request.Type,
                    request.IsDisposable,
                    request.SetFor,
                    request.HasDisposed,
                    request.Amount,
                    request.AmountType,
                    complexId,
                    request.ScheduleType);
            activity.SetUser(request.UserId);

            activity.SetShiftPlan(request.ShiftPlanId);

            repositoryWrapper.SetRepository<Domain.Entities.Tasks.Activity>().Add(activity);
            await repositoryWrapper.SaveChangesAsync(cancellationToken);
            await repositoryWrapper.CommitAsync(cancellationToken);
            return activity.AdaptToLDto();
        }
        catch (Exception)
        {
            await repositoryWrapper.RollBackAsync(cancellationToken);
            throw;
        }
    }
}