﻿namespace Brizco.Repository.Handlers.Activities;

public class DeleteActivityCommandHandler(IRepositoryWrapper repositoryWrapper)
    : IRequestHandler<DeleteActivityCommand, bool>
{
    public async Task<bool> Handle(DeleteActivityCommand request, CancellationToken cancellationToken)
    {
        try
        {
            await repositoryWrapper.BeginTransaction(cancellationToken);
            var task = await repositoryWrapper.SetRepository<Domain.Entities.Tasks.Activity>()
                .TableNoTracking
                .FirstOrDefaultAsync(s => s.Id == request.Id, cancellationToken);
            if (task == null)
                throw new AppException("Task not found", ApiResultStatusCode.NotFound);
            repositoryWrapper.SetRepository<Domain.Entities.Tasks.Activity>()
                .Delete(task);

            await repositoryWrapper.SaveChangesAsync(cancellationToken);
            await repositoryWrapper.CommitAsync(cancellationToken);
            return true;
        }
        catch (Exception )
        {
            await repositoryWrapper.RollBackAsync(cancellationToken);
            throw;
        }
    }
}