﻿namespace Brizco.Repository.Handlers.Complexes;

public class DeleteComplexUserCommandHandler(IRepositoryWrapper repositoryWrapper)
    : IRequestHandler<DeleteComplexUserCommand, bool>
{
    public async Task<bool> Handle(DeleteComplexUserCommand request, CancellationToken cancellationToken)
    {
        try
        {
            await repositoryWrapper.BeginTransaction(cancellationToken);
            var complexUser = await repositoryWrapper.SetRepository<ComplexUser>()
                .TableNoTracking
                .FirstOrDefaultAsync(c => c.ComplexId == request.ComplexId && c.UserId == request.UserId, cancellationToken);
            if (complexUser == null)
                throw new AppException("ComplexUser not found", ApiResultStatusCode.NotFound);
            var complexUserRoles = await repositoryWrapper.SetRepository<ComplexUserRole>().TableNoTracking
                .Where(c => c.ComplexUserId == complexUser.Id)
                .ToListAsync(cancellationToken);
            foreach (var complexUserRole in complexUserRoles)
                repositoryWrapper.SetRepository<ComplexUserRole>().HardDelete(complexUserRole);
            await repositoryWrapper.SaveChangesAsync(cancellationToken);

            repositoryWrapper.SetRepository<ComplexUser>().HardDelete(complexUser);
            await repositoryWrapper.SaveChangesAsync(cancellationToken);
            await repositoryWrapper.CommitAsync(cancellationToken);
            return true;
        }
        catch (Exception)
        {
            await repositoryWrapper.RollBackAsync(cancellationToken);
            throw;
        }
    }
}