﻿namespace Brizco.Repository.Handlers.Positions;

public class UpdatePositionCommandHandler(IRepositoryWrapper repositoryWrapper, ICurrentUserService currentUserService)
    : IRequestHandler<UpdatePositionCommand, bool>
{
    public async Task<bool> Handle(UpdatePositionCommand request, CancellationToken cancellationToken)
    {
        var ent = await repositoryWrapper.SetRepository<Position>()
            .TableNoTracking.FirstOrDefaultAsync(s => s.Id == request.Id, cancellationToken);
        if (ent == null)
            throw new AppException("Postion not found", ApiResultStatusCode.NotFound);

        if (currentUserService.ComplexId == null)
            throw new AppException("ComplexId is null", ApiResultStatusCode.NotFound);
        if (!Guid.TryParse(currentUserService.ComplexId, out Guid complexId))
            throw new AppException("ComplexId is wrong", ApiResultStatusCode.NotFound);

        var newPosition = Position.Create(request.Title,
            request.Description,
            complexId,
            request.SectionId);
        newPosition.Id = request.Id;
        newPosition.CreatedAt = ent.CreatedAt;
        newPosition.CreatedBy = ent.CreatedBy;

        var permissionsDb = await repositoryWrapper.SetRepository<PositionPermission>()
            .TableNoTracking
            .Where(f => f.PositionId == ent.Id)
            .ToListAsync(cancellationToken);

        foreach (var permissionDb in permissionsDb.Where(p=>!request.Permissions.Contains(p.Permission)))
        {
            repositoryWrapper.SetRepository<PositionPermission>()
                .Delete(permissionDb);
            await repositoryWrapper.SaveChangesAsync(cancellationToken);
        }

        foreach (var permission in request.Permissions.Where(p => !permissionsDb.Select(d => d.Permission).Contains(p)))
            newPosition.AddPermission(permission);

        repositoryWrapper.SetRepository<Position>()
            .Update(newPosition);

        await repositoryWrapper.SaveChangesAsync(cancellationToken);

        return true;
    }
}